==> atmosphere-portable-runtime-0.1-sources.jar: org/atmosphere/container/GrizzlyCometSupport.java <==
/*
 * 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */
package org.atmosphere.container;

import com.sun.grizzly.comet.CometEvent;
import java.io.IOException;
import java.util.logging.Level;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.sun.grizzly.comet.CometEngine;
import com.sun.grizzly.comet.CometContext;
import com.sun.grizzly.comet.CometHandler;
import java.util.Map;
import org.atmosphere.cpr.AtmosphereEvent;
import org.atmosphere.cpr.AtmosphereServlet.Action;
import org.atmosphere.cpr.AtmosphereServlet.AtmosphereHandlerWrapper;
import org.atmosphere.cpr.CometSupport;
import org.atmosphere.cpr.WebContainer;
/** 
 * Comet Portable Runtime implementation on top of Grizzly 1.5 and up. 
 *
 * @author Jeanfrancois Arcand
 */
public class GrizzlyCometSupport extends WebContainer implements CometSupport {

    private final static String ATMOSPHERE = "/atmosphere";
    
    private String atmosphereCtx = "";

    public GrizzlyCometSupport(Map<String,AtmosphereHandlerWrapper> atmosphereHandlers){
        super(atmosphereHandlers);
        setWebServerName(getContainerName() + "-" + getContainerDottedVersion());
    }
    
    /**
     * Init Grizzly's {@link CometContext} that will be used to suspend and
     * resume the response.
     * @param sc the {@link ServletContext}
     * @throws javax.servlet.ServletException
     */
    @Override
    public void init(ServletConfig sc) throws ServletException{
        super.init(sc);
        atmosphereCtx =
                sc.getServletContext().getContextPath() + ATMOSPHERE;

        CometEngine cometEngine = CometEngine.getEngine();
        CometContext context = cometEngine.register(atmosphereCtx);
        context.setExpirationDelay(-1);
        if (logger.isLoggable(Level.FINE)){
            logger.fine("Created CometContext for " + atmosphereCtx);
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getContainerName() {
        return "Grizzly";
    }
    
    /**
     * {@inheritDoc}
     */
    public String getContainerDottedVersion() {
        return "1.9.x";
    }

    /**
     * {@inheritDoc}
     */
    public Action service(HttpServletRequest req, HttpServletResponse res)
            throws IOException, ServletException {
        
        CometContext ctx = CometEngine.getEngine().getCometContext(atmosphereCtx);
        VoidCometHandler c = new VoidCometHandler(req, res);
        req.getSession().setAttribute(ATMOSPHERE, c.hashCode());
        
        Action action = suspended(req, res);
        if (action.type == Action.TYPE.SUSPEND) {
            if (logger.isLoggable(Level.FINE)){
                logger.fine("Suspending" + res);
            }

            ctx.setExpirationDelay(action.timeout);                       
            ctx.addCometHandler(c);
       } else if (action.type == Action.TYPE.RESUME){
            if (logger.isLoggable(Level.FINE)){
                logger.fine("Resuming" + res);
            }

            CometHandler handler = ctx
                    .getCometHandler((Integer)req.getAttribute(ATMOSPHERE));
            if (handler != null){
                ctx.resumeCometHandler(handler);
            }
            req.removeAttribute(ATMOSPHERE);
            Action nextAction = resumed(req, res);   
            if (nextAction.type == Action.TYPE.SUSPEND) {
                if (logger.isLoggable(Level.FINE)){
                    logger.fine("Suspending after Resuming" + res);
                }    
                ctx.setExpirationDelay(action.timeout);         
                ctx.addCometHandler(c);          
            }
        }
        return action;
    }
    
    /**
     * Void {@link CometHandler}, which delegate the processing of the 
     * {@link HttpServletResponse} to an {@link AtmosphereEvent}.
     */
    private class VoidCometHandler implements CometHandler{

        HttpServletRequest req;
        HttpServletResponse res;
        
        public VoidCometHandler(HttpServletRequest req, HttpServletResponse res){
            this.req = req;
            this.res = res;
        }
        
        /**
         * {@inheritDoc}
         */
        public void attach(Object o) {
        }

        /**
         * {@inheritDoc}
         */
        public void onEvent(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onInitialize(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onTerminate(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onInterrupt(CometEvent ce) throws IOException {
            try {
                req.removeAttribute(ATMOSPHERE);
                timedout(req, res);
            } catch (ServletException ex) {
                IOException e = new IOException();
                e.initCause(ex);
                throw e;
            }
        }
    }
}

==> atmosphere-portable-runtime-0.2-sources.jar: org/atmosphere/container/GrizzlyCometSupport.java <==
/*
 * 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */
package org.atmosphere.container;

import com.sun.grizzly.comet.CometEvent;
import java.io.IOException;
import java.util.logging.Level;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.sun.grizzly.comet.CometEngine;
import com.sun.grizzly.comet.CometContext;
import com.sun.grizzly.comet.CometHandler;
import org.atmosphere.cpr.AtmosphereEventImpl;
import org.atmosphere.cpr.AtmosphereServlet.Action;
import org.atmosphere.cpr.AtmosphereServlet.AtmosphereConfig;
import org.atmosphere.cpr.CometSupport;
import org.atmosphere.cpr.AsynchronousProcessor;
/** 
 * Comet Portable Runtime implementation on top of Grizzly 1.5 and up. 
 *
 * @author Jeanfrancois Arcand
 */
public class GrizzlyCometSupport extends AsynchronousProcessor implements CometSupport<AtmosphereEventImpl> {

    private final static String ATMOSPHERE = "/atmosphere";
    
    private String atmosphereCtx = "";

    public GrizzlyCometSupport(AtmosphereConfig config){
        super(config);
    }
    
    /**
     * Init Grizzly's {@link CometContext} that will be used to suspend and
     * resume the response.
     * @param sc the {@link ServletContext}
     * @throws javax.servlet.ServletException
     */
    @Override
    public void init(ServletConfig sc) throws ServletException{
        super.init(sc);
        atmosphereCtx =
                sc.getServletContext().getContextPath() + ATMOSPHERE;

        CometEngine cometEngine = CometEngine.getEngine();
        CometContext context = cometEngine.register(atmosphereCtx);
        context.setExpirationDelay(-1);
        if (logger.isLoggable(Level.FINE)){
            logger.fine("Created CometContext for " + atmosphereCtx);
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getContainerName() {
        return "Grizzly";
    }
    
    /**
     * {@inheritDoc}
     */
    public String getContainerDottedVersion() {
        return "1.9.x";
    }

    /**
     * {@inheritDoc}
     */  
    public Action service(HttpServletRequest req, HttpServletResponse res)
            throws IOException, ServletException {
        
        CometContext ctx = CometEngine.getEngine().getCometContext(atmosphereCtx);        
        Action action = suspended(req, res);
        if (action.type == Action.TYPE.SUSPEND) {
            if (logger.isLoggable(Level.FINE)){
                logger.fine("Suspending" + res);
            }
            suspend(ctx, action, req, res);
       } else if (action.type == Action.TYPE.RESUME){
            if (logger.isLoggable(Level.FINE)){
                logger.fine("Resuming" + res);
            }
           
            resume(req,ctx);
            Action nextAction = resumed(req, res);   
            if (nextAction.type == Action.TYPE.SUSPEND) {
                if (logger.isLoggable(Level.FINE)){
                    logger.fine("Suspending after Resuming" + res);
                }    
                suspend(ctx, action, req, res);        
            }
        }
        return action;
    }
    
    /**
     * Suspend the response
     */
    private void suspend(CometContext ctx,Action action,HttpServletRequest req, 
            HttpServletResponse res ){
        VoidCometHandler c = new VoidCometHandler(req, res);
        ctx.setExpirationDelay(action.timeout);                       
        ctx.addCometHandler(c);
        req.getSession().setAttribute(ATMOSPHERE, c.hashCode());
    }
    
    /**
     * Resume the underlying response,
     * @param req an {@link HttpServletRequest}
     * @param ctx a {@link CometContext}
     */
    private void resume(HttpServletRequest req, CometContext ctx){
        CometHandler handler = ctx
            .getCometHandler((Integer)req.getSession().getAttribute(ATMOSPHERE));
        if (handler != null){
            ctx.resumeCometHandler(handler);
        }
        req.removeAttribute(ATMOSPHERE);
    }
    
    /**
     * {@inheritDoc}
     */
    public void action(AtmosphereEventImpl actionEvent){
        if (actionEvent.action().type == Action.TYPE.RESUME){
            CometContext ctx = CometEngine.getEngine().getCometContext(atmosphereCtx);
            resume(actionEvent.getRequest(), ctx);
        }
    }
    
    /**
     * Void {@link CometHandler}, which delegate the processing of the 
     * {@link HttpServletResponse} to an {@link AtmosphereEvent}.
     */
    private class VoidCometHandler implements CometHandler{

        HttpServletRequest req;
        HttpServletResponse res;
        
        public VoidCometHandler(HttpServletRequest req, HttpServletResponse res){
            this.req = req;
            this.res = res;
        }
        
        /**
         * {@inheritDoc}
         */
        public void attach(Object o) {
        }

        /**
         * {@inheritDoc}
         */
        public void onEvent(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onInitialize(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onTerminate(CometEvent ce) throws IOException {
        }

        /**
         * {@inheritDoc}
         */
        public void onInterrupt(CometEvent ce) throws IOException {
            try {
                req.removeAttribute(ATMOSPHERE);
                cancelled(req, res);
            } catch (ServletException ex) {
                logger.log(Level.WARNING,"",ex);
            }
        }
    }
}
